package com.github.jfcloud.gene.flow.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.jfcloud.common.annotation.WebApiLog;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.flow.dto.FlowAuditDto;
import com.github.jfcloud.gene.flow.dto.FlowDetailDto;
import com.github.jfcloud.gene.flow.dto.FlowPageDto;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.FlowInfoService;
import com.github.jfcloud.gene.flow.vo.FlowAuditVo;
import com.github.jfcloud.gene.flow.vo.FlowDetailVo;
import com.github.jfcloud.gene.flow.vo.FlowPageVo;
import com.github.jfcloud.gene.flow.vo.SynProjectVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import javafx.util.Pair;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Tag(name = "基因定制流程")
@RestController
@RequestMapping("/flow")
@RequiredArgsConstructor
public class FlowController {

    private final FlowInfoService flowInfoService;
    private final FlowAuditService flowAuditService;
    private final UserIdNameCache userIdNameCache;

    @Operation(summary = "列表查询")
    @GetMapping("/page")
    public R<Page<FlowPageDto>> page(FlowPageVo vo) {
        return R.ok(flowInfoService.getPage(vo));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    public R<FlowDetailDto> detail(@PathVariable Long id) {
        return R.ok(flowInfoService.getDetail(id));
    }

    @Operation(summary = "状态列表")
    @GetMapping("/statusList")
    public R<List<Pair<String, String>>> statusList() {
        List<Pair<String, String>> pairs = Stream.of(GeneStatusEnum.values())
                .filter(e -> GeneStatusEnum.REJECTED != e)
                .map(x -> new Pair<>(x.getStatus(), x.getDescription()))
                .collect(Collectors.toList());
        return R.ok(pairs);
    }

    @WebApiLog
    @Operation(summary = "新增")
    @PostMapping
    public R save(@Valid @RequestBody FlowDetailVo vo) {
        flowInfoService.saveFlow(vo);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "编辑")
    @PutMapping("/{id}")
    public R update(@PathVariable Long id, @Valid @RequestBody FlowDetailVo vo) {
        flowInfoService.updateFlow(id, vo);
        return R.ok();
    }

    @Operation(summary = "删除")
    @DeleteMapping("/{id}")
    public R delete(@PathVariable Long id) {
        flowInfoService.removeById(id);
        return R.ok();
    }

    @Operation(summary = "提交")
    @PostMapping("/submit/{id}")
    public R submit(@PathVariable Long id) {
        flowInfoService.submitFlow(id);
        return R.ok();
    }

    @Operation(summary = "复制")
    @PostMapping("/copy/{id}")
    public R copy(@PathVariable Long id) {
        flowInfoService.copyFlow(id);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "审核")
    @PostMapping("/audit/{id}")
    public R audit(@PathVariable Long id, @Valid @RequestBody FlowAuditVo vo) {
        flowInfoService.audit(id, vo);
        return R.ok();
    }

    @Operation(summary = "审核记录")
    @GetMapping("/audit/{id}")
    public R<List<FlowAuditDto>> auditRecord(@PathVariable Long id) {
        List<FlowAudit> auditList = flowAuditService.list(new LambdaQueryWrapper<>(FlowAudit.class)
                .eq(FlowAudit::getFlowId, id)
                .eq(FlowAudit::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FlowAudit::getCreateTime));
        if (auditList.isEmpty()) {
            return R.ok(Collections.emptyList());
        }

        List<FlowAuditDto> auditDtos = new ArrayList<>();
        for (FlowAudit audit : auditList) {
            FlowAuditDto auditDto = new FlowAuditDto();
            if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                auditDto = BeanUtil.toBean(audit.getAdditionalData(), FlowAuditDto.class);
            }
            BeanUtil.copyProperties(audit, auditDto);
            auditDto.setCreateName(userIdNameCache.getNicknameByUserId(audit.getCreateBy()));
            auditDto.setFlowStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(audit.getFlowStatus())).getDescription());
            auditDtos.add(auditDto);
        }
        return R.ok(auditDtos);
    }

    @Operation(summary = "执行")
    @PostMapping("/execute/{id}")
    public R execute(@PathVariable Long id, @RequestBody @Valid SynProjectVo vo) {
        flowInfoService.execute(id, vo);
        return R.ok();
    }

    @Operation(summary = "生成文档", hidden = true)
    @PostMapping("/generate/{id}")
    public R generate(@PathVariable Long id) {
        flowInfoService.geneWord(id);
        return R.ok();
    }
}
