package com.github.jfcloud.gene.flow.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.common.data.datascope.DataScope;
import com.github.jfcloud.common.holder.RequestHolder;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.constants.GeneTargetEnum;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.dto.FlowDetailDto;
import com.github.jfcloud.gene.flow.dto.FlowPageDto;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.entity.FlowFileVersion;
import com.github.jfcloud.gene.flow.entity.FlowInfo;
import com.github.jfcloud.gene.flow.mapper.FlowInfoMapper;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.FlowFileVersionService;
import com.github.jfcloud.gene.flow.service.FlowInfoService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.flow.vo.FlowAuditVo;
import com.github.jfcloud.gene.flow.vo.FlowDetailVo;
import com.github.jfcloud.gene.flow.vo.FlowPageVo;
import com.github.jfcloud.gene.flow.vo.SynProjectVo;
import com.github.jfcloud.gene.form.dto.StrainCustomDto;
import com.github.jfcloud.gene.form.dto.StrainPurificationInfoDto;
import com.github.jfcloud.gene.form.entity.StrainCustomInfo;
import com.github.jfcloud.gene.form.entity.StrainPurificationInfo;
import com.github.jfcloud.gene.form.service.StrainCustomInfoService;
import com.github.jfcloud.gene.form.service.StrainPurificationInfoService;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.gene.util.WordUtil;
import com.github.jfcloud.project.api.dto.ProjectFeignDTO;
import com.github.jfcloud.project.api.feign.RemoteProjectService;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.docx4j.Docx4J;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Slf4j
@Service
@RequiredArgsConstructor
public class FlowInfoServiceImpl extends ServiceImpl<FlowInfoMapper, FlowInfo> implements FlowInfoService {

    private final UserIdNameCache userIdNameCache;
    private final StrainCustomInfoService customInfoService;
    private final StrainPurificationInfoService purificationInfoService;
    private final NotifyService notifyService;
    private final WordDataService wordDataService;
    private final FileInfoService fileInfoService;
    private final FlowFileVersionService fileVersionService;
    private final RemoteProjectService remoteProjectService;
    private final FlowAuditService flowAuditService;

    @Override
    public Page<FlowPageDto> getPage(FlowPageVo vo) {
        Page<FlowInfo> pageQuery = new Page<>(vo.getCurrent(), vo.getSize());

        //如果查询待提交，将驳回状态的一并查出
        if (vo.getStatusList().contains(GeneStatusEnum.DRAFT.getStatus())) {
            vo.getStatusList().add(GeneStatusEnum.REJECTED.getStatus());
        }

        LambdaQueryWrapper<FlowInfo> lqw = new LambdaQueryWrapper<>(FlowInfo.class)
                .like(StrUtil.isNotBlank(vo.getProjectName()), FlowInfo::getProjectName, vo.getProjectName())
                .like(StrUtil.isNotBlank(vo.getContractNo()), FlowInfo::getContractNo, vo.getContractNo())
                .like(StrUtil.isNotBlank(vo.getApprovalNo()), FlowInfo::getApprovalNo, vo.getApprovalNo())
                .eq(Objects.nonNull(vo.getProjectLeaderId()), FlowInfo::getProjectLeaderId, vo.getProjectLeaderId())
                .eq(Objects.nonNull(vo.getCreateBy()), FlowInfo::getCreateBy, vo.getCreateBy())
                .in(!vo.getStatusList().isEmpty(), FlowInfo::getStatus, vo.getStatusList())
                .eq(FlowInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByDesc(FlowInfo::getCreateTime);
        Page<FlowInfo> page = baseMapper.selectPageByScope(pageQuery, lqw, DataScope.of());
        if (page.getTotal() < 1) {
            return new Page<>();
        }

        List<FlowPageDto> records = BeanUtil.copyToList(page.getRecords(), FlowPageDto.class);

        records.forEach(item -> {
            //补充申请人姓名
            item.setCreateByName(userIdNameCache.getNicknameByUserId(Long.parseLong(item.getCreateBy())));
            //查询下载链接
            FileVo latestFile = fileVersionService.getLatestFile(item.getId());
            item.setFileUrl(latestFile.getUrl());
            item.setFilename(latestFile.getName());
            item.setStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(item.getStatus())).getDescription());
        });

        Page<FlowPageDto> pageDto = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        return pageDto.setRecords(records);
    }

    @Override
    public FlowDetailDto getDetail(Long id) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");

        FlowDetailDto detail = BeanUtil.copyProperties(flowInfo, FlowDetailDto.class);
        detail.setGeneEditPlName(userIdNameCache.getNicknameByUserId(flowInfo.getGeneEditPlId()));
        detail.setProjectManageName(userIdNameCache.getNicknameByUserId(flowInfo.getProjectManageId()));
        detail.setStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(detail.getStatus())).getDescription());

        StrainCustomDto customDto = customInfoService.getByFlowId(id);
        detail.setCustom(Optional.ofNullable(customDto).orElse(new StrainCustomDto()));
        StrainPurificationInfoDto purificationInfoDto = purificationInfoService.getByFlowId(id);
        detail.setPurification(Optional.ofNullable(purificationInfoDto).orElse(new StrainPurificationInfoDto()));

        //查询下载链接
        FileVo latestFile = fileVersionService.getLatestFile(id);
        detail.setFileUrl(latestFile.getUrl());
        detail.setFilename(latestFile.getName());

        return detail;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveFlow(FlowDetailVo vo) {
        log.info("保存流程 项目名称：{}", vo.getProjectName());
        FlowInfo flowInfo = BeanUtil.copyProperties(vo, FlowInfo.class);
        flowInfo.setId(CustomIdGenerator.nextId());

        boolean targetCheck = false;
        if (vo.getTarget().contains(GeneTargetEnum.CUSTOM.getCode())) {
            targetCheck = true;
            vo.getCustom().setFlowId(flowInfo.getId());
            customInfoService.saveForm(vo.getCustom());
        }

        if (vo.getTarget().contains(GeneTargetEnum.PURIFICATION.getCode())) {
            targetCheck = true;
            vo.getPurification().setFlowId(flowInfo.getId());
            purificationInfoService.saveForm(vo.getPurification());
        }

        Assert.isTrue(targetCheck, "target只能为custom purification");

        flowInfo.setStatus(GeneStatusEnum.DRAFT.getStatus());
        flowInfo.insert();

        geneWord(flowInfo.getId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateFlow(Long id, FlowDetailVo vo) {
        log.info("编辑流程 项目名称：{}", vo.getProjectName());

        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        BeanUtil.copyProperties(vo, flowInfo);

        StringBuilder target = new StringBuilder();
        if (vo.getCustom() == null) {
            customInfoService.remove(new LambdaQueryWrapper<>(StrainCustomInfo.class).eq(StrainCustomInfo::getFlowId, id));
        } else {
            vo.getCustom().setFlowId(id);
            customInfoService.updateForm(vo.getCustom());
            target.append(GeneTargetEnum.CUSTOM);
        }

        if (vo.getPurification() == null) {
            purificationInfoService.remove(new LambdaQueryWrapper<>(StrainPurificationInfo.class).eq(StrainPurificationInfo::getFlowId, id));
        } else {
            vo.getPurification().setFlowId(id);
            purificationInfoService.updateForm(vo.getPurification());

            if (target.length() > 0) {
                target.append(",");
            }
            target.append(GeneTargetEnum.PURIFICATION);
        }

        flowInfo.setTarget(target.toString());
        flowInfo.updateById();

        geneWord(flowInfo.getId());
    }

    @Override
    public void submitFlow(Long id) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isTrue(GeneStatusEnum.SUBMIT_STATUS.contains(flowInfo.getStatus()), "流程状态错误");

        new FlowInfo()
                .setId(id)
                .setStatus(GeneStatusEnum.GENE_EDITING.getStatus())
                .updateById();

        notifyService.notify(id);
    }

    @Override
    public void audit(Long id, FlowAuditVo vo) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isFalse(GeneStatusEnum.SUBMIT_STATUS.contains(flowInfo.getStatus()), "流程状态错误");

        String nextStatus = "";
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(flowInfo.getStatus()));
        switch (statusEnum) {
            case GENE_EDITING:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getGeneEditPlId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.PROJECT_LEADER.getStatus();
                break;
            case PROJECT_LEADER:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getProjectLeaderId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.PROJECT_MANAGEMENT.getStatus();
                break;
            case PROJECT_MANAGEMENT:
                Assert.isTrue(UserUtil.getUserId().equals(flowInfo.getProjectManageId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.COMPLETED.getStatus();
                break;
            default:
                log.info("流程项目 ({}) 状态为{}", flowInfo.getProjectName(), statusEnum.getDescription());
                throw new IllegalArgumentException("流程状态错误");
        }

        //保存审批节点
        FlowAudit flowAudit = new FlowAudit()
                .setFlowId(id)
                .setFlowStatus(flowInfo.getStatus())
                .setAuditResult(vo.getAuditResult())
                .setRemarks(vo.getRemarks())
                .setCreateSign(userIdNameCache.getSignPic(UserUtil.getUserId()));
        if (StrUtil.isNotBlank(vo.getLeaderResult())) {
            JSONObject dataObj = new JSONObject().fluentPut("leaderResult", vo.getLeaderResult());
            flowAudit.setAdditionalData(dataObj.toJSONString());
        }
        flowAudit.insert();

        //审核不通过
        if (StrConstant.NO.equals(vo.getAuditResult())) {
            log.info("流程项目 ({}) 审核不通过", flowInfo.getProjectName());

            new FlowInfo()
                    .setId(id)
                    .setStatus(GeneStatusEnum.REJECTED.getStatus())
                    .updateById();

            geneWord(id);

            notifyService.notify(id);
            return;
        }

        Assert.isTrue(StrConstant.YES.equals(vo.getAuditResult()), "审核结果错误");

        //审核通过
        log.info("流程项目 ({}) 审核通过", flowInfo.getProjectName());

        flowInfo = new FlowInfo();
        //更新项目批准编号、修订号等
        BeanUtil.copyProperties(vo, flowInfo);
        flowInfo.setId(id)
                .setStatus(nextStatus)
                .updateById();

        geneWord(id);

        notifyService.notify(id);
    }

    @Override
    public void execute(Long id, SynProjectVo vo) {
        FlowInfo flowInfo = getById(id);
        Assert.notNull(flowInfo, "基因定制流程不存在");
        Assert.isTrue(GeneStatusEnum.COMPLETED.getStatus().equals(flowInfo.getStatus()), "基因定制流程非完成状态");
        Assert.isNull(flowInfo.getKyId(), "基因定制流程已执行");

        //推送数据到科研项目，并得到科研项目id
        ProjectFeignDTO projectFeignDTO = new ProjectFeignDTO();
        projectFeignDTO.setName(flowInfo.getProjectName());
        projectFeignDTO.setLeader(vo.getProjectLeader());
        projectFeignDTO.setProjectType(vo.getApprovalType());

        //推送申请人nickname
        String nickname = userIdNameCache.getNicknameByUserId(vo.getProjectLeader());
        projectFeignDTO.setLeaders(nickname);

        log.info("推送科研项目：flowId={}, param={}", id, JSON.toJSONString(projectFeignDTO));

        R<Long> longR = remoteProjectService.initProject(projectFeignDTO);
        if (!longR.isOk()) {
            log.error("projectId={}推送科研失败，{}", id, longR);
        }
        Assert.isTrue(longR.isOk(), "操作失败，科研" + longR);
        Long kyId = longR.getData();
        new FlowInfo()
                .setId(id)
                .setKyId(kyId)
                .updateById();
    }

    /**
     * 生成word
     */
    @SneakyThrows
    public void geneWord(Long flowId) {
        FlowDetailDto detail = getDetail(flowId);
        FlowInfo flowInfo = getById(flowId);

        //文件名称：项目名称-负责人-申请日期-随机四位.docx
        String createDate = DateUtil.format(flowInfo.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("立项报告-动物%s申请表-%s-%s-%s-%s.docx",
                GeneTargetEnum.getNameByCode(flowInfo.getTarget()),
                detail.getProjectName(), detail.getProjectLeaderName(), createDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 项目名称：{} 文件名称：{}", detail.getProjectName(), fileName);

        JSONObject dataMap = JSON.parseObject(JSON.toJSONString(detail));
        wordDataService.fillDocCreator(dataMap);
        dataMap.put("createDateStr", createDate);
        dataMap.put("createBySign", userIdNameCache.getSignBase64(Long.valueOf(flowInfo.getCreateBy())));

        dataMap.put("approvalNo", WordUtil.padString(flowInfo.getApprovalNo(), 14));
        dataMap.put("revisionNo", WordUtil.padString(flowInfo.getRevisionNo(), 14));
        dataMap.put("receiptDate", WordUtil.padString(flowInfo.getReceiptDate(), 12));
        dataMap.put("approvalDate", WordUtil.padString(flowInfo.getApprovalDate(), 12));
        dataMap.put("animalEnrollment", WordUtil.padString(flowInfo.getAnimalEnrollment(), 12));
        dataMap.put("finalSampling", WordUtil.padString(flowInfo.getFinalSampling(), 12));

        //查询审核记录
        List<FlowAudit> flowAudits = flowAuditService.list(new LambdaQueryWrapper<>(FlowAudit.class)
                .eq(FlowAudit::getFlowId, flowId)
                .eq(FlowAudit::getDeleted, WhetherEnum.NO.getCode())
                .orderByDesc(FlowAudit::getCreateTime));
        //项目管理部审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectManagementDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    dataMap.put("projectManagementSign", wordDataService.getBase64(audit.getCreateSign()));
                });
        //项目负责人审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_LEADER.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectLeaderDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    dataMap.put("projectLeaderSign", wordDataService.getBase64(audit.getCreateSign()));
                    if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                        JSONObject additionObj = JSON.parseObject(audit.getAdditionalData());
                        dataMap.put("leaderResult", StrConstant.YES.equals(additionObj.getOrDefault("leaderResult", "")));
                    }
                });
        //基因编辑部门审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.GENE_EDITING.getStatus().equals(audit.getFlowStatus()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("geneEditingResult", StrConstant.YES.equals(audit.getAuditResult()));
                    dataMap.put("geneEditingRemarks", audit.getRemarks());
                });


        String modelName = "geneCustom.ftl";

        try (InputStream inputStream = WordUtil.exportWord(modelName, dataMap);
             ByteArrayOutputStream out = new ByteArrayOutputStream()) {
            //转化为docx文件
            WordprocessingMLPackage pkg = Docx4J.load(inputStream);
            Docx4J.save(pkg, out);

            //上传至文件服务器
            byte[] byteArray = out.toByteArray();
            try (ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(byteArray)) {
                FileVo fileVo = fileInfoService.uploadFileWithFileName(byteArrayInputStream, fileName);

                HttpServletRequest request = RequestHolder.getRequest();
                String uri = request.getMethod() + " " + request.getRequestURI();

                new FlowFileVersion()
                        .setFlowId(flowInfo.getId())
                        .setFlowStatus(flowInfo.getStatus())
                        .setDataMapWithoutImg(dataMap.toJSONString())
                        .setFileMd5(DigestUtil.md5Hex(byteArray))
                        .setFilename(fileVo.getName())
                        .setFilepath(fileVo.getUrl())
                        .setUri(uri)
                        .setTemplateName(modelName)
                        .insert();
            }
        }


    }

    @Override
    public void copyFlow(Long id) {
        FlowDetailDto detail = getDetail(id);
        FlowDetailVo vo = BeanUtil.copyProperties(detail, FlowDetailVo.class);
        vo.setProjectName(vo.getProjectName() + "(1)");
        saveFlow(vo);
    }
}
