package com.github.jfcloud.gene.flow.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.admin.api.sys.dto.message.TextMessageParam;
import com.github.jfcloud.admin.api.sys.feign.RemoteNoticeService;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.entity.FlowInfo;
import com.github.jfcloud.gene.flow.mapper.FlowAuditMapper;
import com.github.jfcloud.gene.flow.mapper.FlowInfoMapper;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Objects;

@Slf4j
@Service
@RequiredArgsConstructor
public class NotifyServiceImpl implements NotifyService {

    private final FlowInfoMapper flowInfoMapper;
    private final FlowAuditMapper flowAuditMapper;
    private final DBSystemPropertiesService systemPropertiesService;
    private final RemoteNoticeService remoteNoticeService;
    private final UserIdNameCache userIdNameCache;

    @Override
    public void notify(Long flowId) {
        FlowInfo flowInfo = flowInfoMapper.selectById(flowId);
        if (flowInfo == null) {
            return;
        }

        //查询流程前一个状态
        FlowAudit previousAudit = flowAuditMapper.selectOne(new LambdaQueryWrapper<>(FlowAudit.class)
                .eq(FlowAudit::getFlowId, flowId)
                .eq(FlowAudit::getDeleted, WhetherEnum.NO.getCode())
                .orderByDesc(FlowAudit::getCreateTime)
                .last("limit 1"));
        String previousStatus = "";
        String previousUser = "";
        if (previousAudit != null) {
            GeneStatusEnum statusEnum = GeneStatusEnum.getByStatus(previousAudit.getFlowStatus());
            previousStatus = statusEnum != null ? statusEnum.getDescription() : "";
            previousUser = userIdNameCache.getNicknameByUserId(previousAudit.getCreateBy());
        }

        Long userId;
        String dingMsg;
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(flowInfo.getStatus()));
        switch (statusEnum) {
            case REJECTED:
                //驳回通知申请人
                userId = Long.valueOf(flowInfo.getCreateBy());
                dingMsg = String.format("【基因定制/%s】【%s/%s】不同意，请修正后再提交", flowInfo.getProjectName(), previousStatus, previousUser);
                break;
            case GENE_EDITING:
                userId = flowInfo.getGeneEditPlId();
                dingMsg = String.format("【基因定制/%s】【%s】已提交，请进行审查", flowInfo.getProjectName(), userIdNameCache.getNicknameByUserId(flowInfo.getCreateBy()));
                break;
            case PROJECT_LEADER:
                userId = flowInfo.getProjectLeaderId();
                dingMsg = String.format("【基因定制/%s】【%s/%s】已同意，请进行审查", flowInfo.getProjectName(), previousStatus, previousUser);
                break;
            case PROJECT_MANAGEMENT:
                userId = flowInfo.getProjectManageId();
                dingMsg = String.format("【基因定制/%s】【%s/%s】已同意，请进行审查", flowInfo.getProjectName(), previousStatus, previousUser);
                break;
            case COMPLETED:
                //执行
                userId = Long.valueOf(flowInfo.getCreateBy());
                dingMsg = String.format("【基因定制/%s】【%s/%s】已同意，请执行项目", flowInfo.getProjectName(), previousStatus, previousUser);
                break;
            default:
                log.error("流程项目 ({}) 状态异常 {}", flowInfo.getProjectName(), statusEnum.getDescription());
                return;
        }

        //查询是否需要钉钉通知
        String key = String.format("gene.%s.dingding.enable", flowInfo.getStatus());
        if (Boolean.parseBoolean(systemPropertiesService.getValue(key))) {
            MessageUserDTO messageUserDTO = new MessageUserDTO();
            messageUserDTO.setUserId(userId);
            sendDingding(dingMsg, messageUserDTO);
        }

        //查询是否需要邮件通知，目前邮件通知随钉钉消息一起发送
//        key = String.format("gene.%s.email.enable", flowInfo.getStatus());
    }

    /**
     * 发送钉钉消息
     *
     * @param msg
     * @param userList
     */
    private void sendDingding(String msg, MessageUserDTO... user) {
        TextMessageParam param = new TextMessageParam(new ArrayList<>(Arrays.asList(user)), msg, "基因定制");
        log.info("【消息通知】发送：{}", JSON.toJSONString(param));

        try {
            remoteNoticeService.sendTextMessage(param);
        } catch (Exception e) {
            //发送失败，异常不向上抛出，不影响业务流程
            log.error("【消息通知】发送失败：{}", e.getMessage());
        }
    }
}
